@@ -6,6 +6,7 @@
 #' after applying a particular scaling to it.
 #'
 #' @inheritParams bigstatsr-package
+#'
 #' @inheritSection bigstatsr-package Matrix parallelization
 #'
 #' @return A temporary [FBM][FBM-class], with the following two attributes:
@@ -46,11 +47,12 @@ big_crossprodSelf <- function(
     sums[ind1]  <- colSums(tmp1)
 
     K[ind1, ind1] <- crossprod(tmp1)
-    for (i in seq_len(j - 1)) {
-      ind2 <- seq2(intervals[i, ])
-      tmp2 <- X[ind.row, ind.col[ind2]]
 
-      K.part <- crossprod(tmp2, tmp1)
+    next_lower <- intervals[j, "upper"] + 1L
+    if (next_lower <= m) {
+      ind2 <- next_lower:m
+      K.part <- big_cprodMat(X, tmp1, ind.row, ind.col[ind2],
+                             block.size = block.size) # TODO: add ncores
       K[ind2, ind1] <- K.part
       K[ind1, ind2] <- t(K.part)
     }
