@@ -302,7 +302,7 @@ test_that("code is used for FBM.code256", {
   # simulating some data
   N <- 230
   M <- 730
-  X <- FBM.code256(N, M, init = rnorm(N * M, sd = 5), code = rnorm(256))
+  X <- FBM.code256(N, M, init = round(100 + rnorm(N * M, sd = 5)), code = rnorm(256))
   y01 <- as.numeric((rowSums(X[, 1:10]) + rnorm(N)) > 0)
   covar <- matrix(rnorm(N * 3), N)
 
